/*
 * DISCLAIMER AND LIMITATION OF LIABILITY: Opto Engineering does not make or
 * give any representation or warranty with respect to the usefulness or the
 * efficiency of this software, it being understood that the degree of success
 * with which equipment, software, modifications, and other materials can be
 * applied to data processing is dependent upon many factors, many of which
 * are not under Opto Engineering's control.  ACCORDINGLY, THIS SOFTWARE IS
 * PROVIDED 'AS IS' WITHOUT EXPRESS OR IMPLIED WARRANTIES, INCLUDING NO
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NONINFRINGEMENT.  THIS SOFTWARE IS PROVIDED GRATUITOUSLY AND, ACCORDINGLY,
 * OPTO ENGINEERING SHALL NOT BE LIABLE UNDER ANY THEORY FOR ANY DAMAGES
 * SUFFERED BY YOU OR ANY USER OF THE SOFTWARE.  OPTO ENGINEERING WILL NOT
 * SUPPORT THIS SOFTWARE AND IS UNDER NO OBLIGATION TO ISSUE UPDATES TO THIS
 * SOFTWARE.
 *
 * WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, NEITHER OPTO ENGINEERING
 * NOR ITS SUPPLIERS SHALL BE LIABLE FOR (a) INCIDENTAL, CONSEQUENTIAL,
 * SPECIAL OR INDIRECT DAMAGES OF ANY SORT, WHETHER ARISING IN TORT, CONTRACT
 * OR OTHERWISE, EVEN IF OPTO ENGINEERING HAS BEEN INFORMED OF THE POSSIBILITY
 * OF SUCH DAMAGES, OR (b) FOR ANY CLAIM BY ANY OTHER PARTY.  SOME STATES DO
 * NOT ALLOW THE EXCLUSION OR LIMITATION OF INCIDENTAL OR CONSEQUENTIAL
 * DAMAGES, SO THIS LIMITATION AND EXCLUSION MAY NOT APPLY TO YOU.
 *
 * Written with Microsoft Visual C++ 2013 Express.
 *
 * Date: 25 APR 2018 - Version: 1.0
 * Author: Sergio Sigala
 */

#include "stdafx.h"

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "modbus.h"
#include "dev_ltdve2ch.h"

/* read all the general purpose registers */
int ltdve2ch_gen_reg_rd(modbus_t *p_modbus, ltdve2ch_reg_rd_t *p_reg, int print, int read_all_regs)
{
	int err = 0;

	if (p_modbus == NULL) { err = 1; printf("bad parameter p_modbus\n"); goto exit; }
	if (p_reg == NULL) { err = 1; printf("bad parameter p_reg\n"); goto exit; }

	if ((err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DEVICE_TYPE, 1, &p_reg->device_type, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (print) printf("DEVICE_TYPE\t\t\t = %d\n", p_reg->device_type);

	if ((err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_BOOT_VERSION, 1, &p_reg->boot_version, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (print) printf("BOOT_VERSION\t\t\t = %d\n", p_reg->boot_version);

	if ((err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_MCU_VERSION, 1, &p_reg->mcu_version, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (print) printf("MCU_VERSION\t\t\t = %d\n", p_reg->mcu_version);

	if ((err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_FPGA_VERSION, 1, &p_reg->fpga_version, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (print) printf("FPGA_VERSION\t\t\t = %d\n", p_reg->fpga_version);

	if ((err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_BOARD_VERSION, 1, &p_reg->board_version, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (print) printf("BOARD_VERSION\t\t\t = %d\n", p_reg->board_version);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OSC_PERIOD, 1, &p_reg->osc_period, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OSC_PERIOD\t\t\t = %d\n", p_reg->osc_period);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_FILTER_SEL0, 1, &p_reg->filter_sel0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("FILTER_SEL0\t\t\t = %d\n", p_reg->filter_sel0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_FILTER_SEL1, 1, &p_reg->filter_sel1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("FILTER_SEL1\t\t\t = %d\n", p_reg->filter_sel1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL0, 1, &p_reg->input_sel0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("INPUT_SEL0\t\t\t = 0x%04X\n", p_reg->input_sel0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL1, 1, &p_reg->input_sel1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("INPUT_SEL1\t\t\t = 0x%04X\n", p_reg->input_sel1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL2, 1, &p_reg->input_sel2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("INPUT_SEL2\t\t\t = 0x%04X\n", p_reg->input_sel2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL3, 1, &p_reg->input_sel3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("INPUT_SEL3\t\t\t = 0x%04X\n", p_reg->input_sel3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE0, 1, &p_reg->gen_dly_base0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_BASE0\t\t\t = %d\n", p_reg->gen_dly_base0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT0, 1, &p_reg->gen_dly_cnt0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_COUNT0\t\t = %d\n", p_reg->gen_dly_cnt0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE0, 1, &p_reg->gen_wdt_base0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_BASE0\t\t\t = %d\n", p_reg->gen_wdt_base0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT0, 1, &p_reg->gen_wdt_cnt0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_COUNT0\t\t = %d\n", p_reg->gen_wdt_cnt0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE1, 1, &p_reg->gen_dly_base1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_BASE1\t\t\t = %d\n", p_reg->gen_dly_base1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT1, 1, &p_reg->gen_dly_cnt1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_COUNT1\t\t = %d\n", p_reg->gen_dly_cnt1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE1, 1, &p_reg->gen_wdt_base1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_BASE1\t\t\t = %d\n", p_reg->gen_wdt_base1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT1, 1, &p_reg->gen_wdt_cnt1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_COUNT1\t\t = %d\n", p_reg->gen_wdt_cnt1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE2, 1, &p_reg->gen_dly_base2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_BASE2\t\t\t = %d\n", p_reg->gen_dly_base2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT2, 1, &p_reg->gen_dly_cnt2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_COUNT2\t\t = %d\n", p_reg->gen_dly_cnt2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE2, 1, &p_reg->gen_wdt_base2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_BASE2\t\t\t = %d\n", p_reg->gen_wdt_base2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT2, 1, &p_reg->gen_wdt_cnt2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_COUNT2\t\t = %d\n", p_reg->gen_wdt_cnt2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE3, 1, &p_reg->gen_dly_base3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_BASE3\t\t\t = %d\n", p_reg->gen_dly_base3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT3, 1, &p_reg->gen_dly_cnt3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_DELAY_COUNT3\t\t = %d\n", p_reg->gen_dly_cnt3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE3, 1, &p_reg->gen_wdt_base3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_BASE3\t\t\t = %d\n", p_reg->gen_wdt_base3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT3, 1, &p_reg->gen_wdt_cnt3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_WIDTH_COUNT3\t\t = %d\n", p_reg->gen_wdt_cnt3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI0, 1, &p_reg->output_sel_hi0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_HI0\t\t\t = 0x%04X\n", p_reg->output_sel_hi0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO0, 1, &p_reg->output_sel_lo0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_LO0\t\t\t = 0x%04X\n", p_reg->output_sel_lo0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI1, 1, &p_reg->output_sel_hi1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_HI1\t\t\t = 0x%04X\n", p_reg->output_sel_hi1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO1, 1, &p_reg->output_sel_lo1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_LO1\t\t\t = 0x%04X\n", p_reg->output_sel_lo1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI8, 1, &p_reg->output_sel_hi8, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_HI8\t\t\t = 0x%04X\n", p_reg->output_sel_hi8);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO8, 1, &p_reg->output_sel_lo8, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_LO8\t\t\t = 0x%04X\n", p_reg->output_sel_lo8);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI9, 1, &p_reg->output_sel_hi9, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_HI9\t\t\t = 0x%04X\n", p_reg->output_sel_hi9);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO9, 1, &p_reg->output_sel_lo9, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("OUTPUT_SEL_LO9\t\t\t = 0x%04X\n", p_reg->output_sel_lo9);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_ON0, 1, &p_reg->prt_cnt_on0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_CNT_ON0\t\t\t = %d\n", p_reg->prt_cnt_on0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_ON0, 1, &p_reg->prt_ena_on0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_ENA_ON0\t\t\t = %d\n", p_reg->prt_ena_on0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_OFF0, 1, &p_reg->prt_cnt_off0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_CNT_OFF0\t\t\t = %d\n", p_reg->prt_cnt_off0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_OFF0, 1, &p_reg->prt_ena_off0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_ENA_OFF0\t\t\t = %d\n", p_reg->prt_ena_off0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_ON1, 1, &p_reg->prt_cnt_on1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_CNT_ON1\t\t\t = %d\n", p_reg->prt_cnt_on1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_ON1, 1, &p_reg->prt_ena_on1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_ENA_ON1\t\t\t = %d\n", p_reg->prt_ena_on1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_OFF1, 1, &p_reg->prt_cnt_off1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_CNT_OFF1\t\t\t = %d\n", p_reg->prt_cnt_off1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_OFF1, 1, &p_reg->prt_ena_off1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("PRT_ENA_OFF1\t\t\t = %d\n", p_reg->prt_ena_off1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_RANGE0, 1, &p_reg->cur_range0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_RANGE0\t\t\t = %d\n", p_reg->cur_range0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_VALUE0, 1, &p_reg->cur_value0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_VALUE0\t\t\t = %d => %.3f A\n", p_reg->cur_value0, p_reg->cur_value0 / 1000.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_RANGE1, 1, &p_reg->cur_range1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_RANGE1\t\t\t = %d\n", p_reg->cur_range1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_VALUE1, 1, &p_reg->cur_value1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_VALUE1\t\t\t = %d => %.3f A\n", p_reg->cur_value1, p_reg->cur_value1 / 1000.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_RS485_MODBUS_ADDR, 1, &p_reg->rs485_modbus_addr, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("RS485_MODBUS_ADDR\t\t = %d\n", p_reg->rs485_modbus_addr);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_RS485_LINE_SPEED, 1, &p_reg->rs485_line_speed, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("RS485_LINE_SPEED\t\t = %d\n", p_reg->rs485_line_speed);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_RS485_LINE_PARITY, 1, &p_reg->rs485_line_parity, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("RS485_LINE_PARITY\t\t = %d\n", p_reg->rs485_line_parity);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_MAC_ADDR0, 1, &p_reg->eth_mac_addr0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_MAC_ADDR0\t\t\t = 0x%04X\n", p_reg->eth_mac_addr0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_MAC_ADDR1, 1, &p_reg->eth_mac_addr1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_MAC_ADDR1\t\t\t = 0x%04X\n", p_reg->eth_mac_addr1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_MAC_ADDR2, 1, &p_reg->eth_mac_addr2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_MAC_ADDR2\t\t\t = 0x%04X\n", p_reg->eth_mac_addr2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME0, 1, &p_reg->eth_hostname0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME0\t\t\t = 0x%04X\n", p_reg->eth_hostname0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME1, 1, &p_reg->eth_hostname1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME1\t\t\t = 0x%04X\n", p_reg->eth_hostname1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME2, 1, &p_reg->eth_hostname2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME2\t\t\t = 0x%04X\n", p_reg->eth_hostname2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME3, 1, &p_reg->eth_hostname3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME3\t\t\t = 0x%04X\n", p_reg->eth_hostname3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME4, 1, &p_reg->eth_hostname4, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME4\t\t\t = 0x%04X\n", p_reg->eth_hostname4);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME5, 1, &p_reg->eth_hostname5, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME5\t\t\t = 0x%04X\n", p_reg->eth_hostname5);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME6, 1, &p_reg->eth_hostname6, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME6\t\t\t = 0x%04X\n", p_reg->eth_hostname6);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME7, 1, &p_reg->eth_hostname7, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_HOSTNAME7\t\t\t = 0x%04X\n", p_reg->eth_hostname7);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_DHCP_ENABLE, 1, &p_reg->eth_dhcp_enable, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_DHCP_ENABLE\t\t\t = %d\n", p_reg->eth_dhcp_enable);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_IP_ADDR_HI, 1, &p_reg->eth_ip_addr_hi, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_IP_ADDR_HI\t\t\t = 0x%04X\n", p_reg->eth_ip_addr_hi);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_IP_ADDR_LO, 1, &p_reg->eth_ip_addr_lo, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_IP_ADDR_LO\t\t\t = 0x%04X\n", p_reg->eth_ip_addr_lo);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_SUBNET_MASK_HI, 1, &p_reg->eth_subnet_mask_hi, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_SUBNET_MASK_HI\t\t = 0x%04X\n", p_reg->eth_subnet_mask_hi);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_SUBNET_MASK_LO, 1, &p_reg->eth_subnet_mask_lo, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_SUBNET_MASK_LO\t\t = 0x%04X\n", p_reg->eth_subnet_mask_lo);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_DEF_GATEWAY_HI, 1, &p_reg->eth_def_gateway_hi, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_DEF_GATEWAY_HI\t\t = 0x%04X\n", p_reg->eth_def_gateway_hi);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_DEF_GATEWAY_LO, 1, &p_reg->eth_def_gateway_lo, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_DEF_GATEWAY_LO\t\t = 0x%04X\n", p_reg->eth_def_gateway_lo);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_PRI_DNS_HI, 1, &p_reg->eth_pri_dns_hi, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_PRI_DNS_HI\t\t\t = 0x%04X\n", p_reg->eth_pri_dns_hi);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_PRI_DNS_LO, 1, &p_reg->eth_pri_dns_lo, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_PRI_DNS_LO\t\t\t = 0x%04X\n", p_reg->eth_pri_dns_lo);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_SEC_DNS_HI, 1, &p_reg->eth_sec_dns_hi, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_SEC_DNS_HI\t\t\t = 0x%04X\n", p_reg->eth_sec_dns_hi);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_SEC_DNS_LO, 1, &p_reg->eth_sec_dns_lo, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_SEC_DNS_LO\t\t\t = 0x%04X\n", p_reg->eth_sec_dns_lo);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_MODBUS_ADDR, 1, &p_reg->eth_modbus_addr, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_MODBUS_ADDR\t\t\t = %d\n", p_reg->eth_modbus_addr);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_MODBUS_TCP_PORT, 1, &p_reg->eth_modbus_tcp_port, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_MODBUS_TCP_PORT\t\t = %d\n", p_reg->eth_modbus_tcp_port);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ETH_MODBUS_UDP_PORT, 1, &p_reg->eth_modbus_udp_port, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ETH_MODBUS_UDP_PORT\t\t = %d\n", p_reg->eth_modbus_udp_port);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD0, 1, &p_reg->web_password0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("WEB_PASSWORD0\t\t\t = 0x%04X\n", p_reg->web_password0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD1, 1, &p_reg->web_password1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("WEB_PASSWORD1\t\t\t = 0x%04X\n", p_reg->web_password1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD2, 1, &p_reg->web_password2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("WEB_PASSWORD2\t\t\t = 0x%04X\n", p_reg->web_password2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD3, 1, &p_reg->web_password3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("WEB_PASSWORD3\t\t\t = 0x%04X\n", p_reg->web_password3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CONVERTER_TEMPERATURE, 1, &p_reg->converter_temperature, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CONVERTER_TEMPERATURE\t\t = %d => %.1f C\n", p_reg->converter_temperature, p_reg->converter_temperature / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DRIVER_TEMPERATURE, 1, &p_reg->driver_temperature, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("DRIVER_TEMPERATURE\t\t = %d => %.1f C\n", p_reg->driver_temperature, p_reg->driver_temperature / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_REMOTE_TEMPERATURE0, 1, &p_reg->remote_temperature0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("REMOTE_TEMPERATURE0\t\t = %d => %.1f C\n", p_reg->remote_temperature0, p_reg->remote_temperature0 / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_REMOTE_TEMPERATURE1, 1, &p_reg->remote_temperature1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("REMOTE_TEMPERATURE1\t\t = %d => %.1f C\n", p_reg->remote_temperature1, p_reg->remote_temperature1 / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_SUPPLY_VOLTAGE, 1, &p_reg->supply_voltage, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("SUPPLY_VOLTAGE\t\t\t = %d => %.1f V\n", p_reg->supply_voltage, p_reg->supply_voltage / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_MEASURED_CURRENT0, 1, &p_reg->measured_current0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("MEASURED_CURRENT0\t\t = %d => %.2f A\n", p_reg->measured_current0, p_reg->measured_current0 / 1000.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_MEASURED_CURRENT1, 1, &p_reg->measured_current1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("MEASURED_CURRENT1\t\t = %d => %.2f A\n", p_reg->measured_current1, p_reg->measured_current1 / 1000.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_MEASURED_VOLTAGE0, 1, &p_reg->measured_voltage0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("MEASURED_VOLTAGE0\t\t = %d => %.1f V\n", p_reg->measured_voltage0, p_reg->measured_voltage0 / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_MEASURED_VOLTAGE1, 1, &p_reg->measured_voltage1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("MEASURED_VOLTAGE1\t\t = %d => %.1f V\n", p_reg->measured_voltage1, p_reg->measured_voltage1 / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_ERROR_WORD, 1, &p_reg->error_word, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("ERROR_WORD\t\t\t = 0x%04X\n", p_reg->error_word);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE0, 1, &p_reg->gen_hld_base0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_BASE0\t\t\t = %d\n", p_reg->gen_hld_base0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT0, 1, &p_reg->gen_hld_cnt0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_COUNT0\t\t\t = %d\n", p_reg->gen_hld_cnt0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE1, 1, &p_reg->gen_hld_base1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_BASE1\t\t\t = %d\n", p_reg->gen_hld_base1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT1, 1, &p_reg->gen_hld_cnt1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_COUNT1\t\t\t = %d\n", p_reg->gen_hld_cnt1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE2, 1, &p_reg->gen_hld_base2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_BASE2\t\t\t = %d\n", p_reg->gen_hld_base2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT2, 1, &p_reg->gen_hld_cnt2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_COUNT2\t\t\t = %d\n", p_reg->gen_hld_cnt2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE3, 1, &p_reg->gen_hld_base3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_BASE3\t\t\t = %d\n", p_reg->gen_hld_base3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT3, 1, &p_reg->gen_hld_cnt3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_HOLD_COUNT3\t\t\t = %d\n", p_reg->gen_hld_cnt3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL0, 1, &p_reg->gen_edge_sel0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_EDGE_SEL0\t\t\t = %d\n", p_reg->gen_edge_sel0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL1, 1, &p_reg->gen_edge_sel1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_EDGE_SEL1\t\t\t = %d\n", p_reg->gen_edge_sel1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL2, 1, &p_reg->gen_edge_sel2, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_EDGE_SEL2\t\t\t = %d\n", p_reg->gen_edge_sel2);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL3, 1, &p_reg->gen_edge_sel3, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("GEN_EDGE_SEL3\t\t\t = %d\n", p_reg->gen_edge_sel3);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_INP_POL_SEL0, 1, &p_reg->inp_pol_sel0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("POLARITY_SEL0\t\t\t = %d => %s\n", p_reg->inp_pol_sel0, p_reg->inp_pol_sel0 ? "Inverted" : "Normal");

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_INP_POL_SEL1, 1, &p_reg->inp_pol_sel1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("POLARITY_SEL1\t\t\t = %d => %s\n", p_reg->inp_pol_sel1, p_reg->inp_pol_sel1 ? "Inverted" : "Normal");

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DRIVE_TIME0, 1, &p_reg->drive_time0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("DRIVE_TIME0\t\t\t = %d ms\n", p_reg->drive_time0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DRIVE_TIME1, 1, &p_reg->drive_time1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("DRIVE_TIME1\t\t\t = %d ms\n", p_reg->drive_time1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_RED_DLY0, 1, &p_reg->cur_red_delay0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_RED_DELAY0\t\t\t = %d\n", p_reg->cur_red_delay0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_RED_VAL0, 1, &p_reg->cur_red_value0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_RED_VALUE0\t\t\t = %d\n", p_reg->cur_red_value0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_RED_DLY1, 1, &p_reg->cur_red_delay1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_RED_DELAY1\t\t\t = %d\n", p_reg->cur_red_delay1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CUR_RED_VAL1, 1, &p_reg->cur_red_value1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CUR_RED_VALUE1\t\t\t = %d\n", p_reg->cur_red_value1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_SOFTWARE_TRIGGER, 1, &p_reg->software_trigger, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("SOFTWARE_TRIGGER\t\t = %d\n", p_reg->software_trigger);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CONVERTER_MODE, 1, &p_reg->converter_mode, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CONVERTER_MODE\t\t\t = %d\n", p_reg->converter_mode);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CONVERTER_MAXIMUM_VLT, 1, &p_reg->converter_maximum_voltage, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CONVERTER_MAXIMUM_VOLTAGE\t = %d V\n", p_reg->converter_maximum_voltage);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CONVERTER_PRESET_VLT, 1, &p_reg->converter_preset_voltage, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CONVERTER_PRESET_VOLTAGE\t = %d V\n", p_reg->converter_preset_voltage);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DRIVER_MODE, 1, &p_reg->driver_mode, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("DRIVER_MODE\t\t\t = %d\n", p_reg->driver_mode);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DIGIPOT_VALUE, 1, &p_reg->digipot_value, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("DIGIPOT_VALUE\t\t\t = %d\n", p_reg->digipot_value);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CONVERTER_VOLTAGE, 1, &p_reg->converter_voltage, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CONVERTER_VOLTAGE\t\t = %d => %.1f V\n", p_reg->converter_voltage, p_reg->converter_voltage / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_DRIVER_VOLTAGE, 1, &p_reg->driver_voltage, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("DRIVER_VOLTAGE\t\t\t = %d => %.1f V\n", p_reg->driver_voltage, p_reg->driver_voltage / 10.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_SUPPLY_CURRENT, 1, &p_reg->supply_current, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("SUPPLY_CURRENT\t\t\t = %d => %.2f A\n", p_reg->supply_current, p_reg->supply_current / 1000.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CONVERTER_CURRENT, 1, &p_reg->converter_current, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CONVERTER_CURRENT\t\t = %d => %.2f A\n", p_reg->converter_current, p_reg->converter_current / 1000.0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_MUL0, 1, &p_reg->cal_gen_cur_low_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_LOW_MUL0\t\t = %d\n", p_reg->cal_gen_cur_low_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_ADD0, 1, &p_reg->cal_gen_cur_low_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_LOW_ADD0\t\t = %d\n", p_reg->cal_gen_cur_low_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_MUL1, 1, &p_reg->cal_gen_cur_low_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_LOW_MUL1\t\t = %d\n", p_reg->cal_gen_cur_low_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_ADD1, 1, &p_reg->cal_gen_cur_low_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_LOW_ADD1\t\t = %d\n", p_reg->cal_gen_cur_low_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_MUL0, 1, &p_reg->cal_gen_cur_high_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_HIGH_MUL0\t\t = %d\n", p_reg->cal_gen_cur_high_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_ADD0, 1, &p_reg->cal_gen_cur_high_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_HIGH_ADD0\t\t = %d\n", p_reg->cal_gen_cur_high_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_MUL1, 1, &p_reg->cal_gen_cur_high_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_HIGH_MUL1\t\t = %d\n", p_reg->cal_gen_cur_high_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_ADD1, 1, &p_reg->cal_gen_cur_high_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_GEN_CUR_HIGH_ADD1\t\t = %d\n", p_reg->cal_gen_cur_high_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_MUL0, 1, &p_reg->cal_mes_cur_low_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_LOW_MUL0\t\t = %d\n", p_reg->cal_mes_cur_low_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_ADD0, 1, &p_reg->cal_mes_cur_low_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_LOW_ADD0\t\t = %d\n", p_reg->cal_mes_cur_low_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_MUL1, 1, &p_reg->cal_mes_cur_low_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_LOW_MUL1\t\t = %d\n", p_reg->cal_mes_cur_low_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_ADD1, 1, &p_reg->cal_mes_cur_low_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_LOW_ADD1\t\t = %d\n", p_reg->cal_mes_cur_low_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_MUL0, 1, &p_reg->cal_mes_cur_high_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_HIGH_MUL0\t\t = %d\n", p_reg->cal_mes_cur_high_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_ADD0, 1, &p_reg->cal_mes_cur_high_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_HIGH_ADD0\t\t = %d\n", p_reg->cal_mes_cur_high_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_MUL1, 1, &p_reg->cal_mes_cur_high_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_HIGH_MUL1\t\t = %d\n", p_reg->cal_mes_cur_high_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_ADD1, 1, &p_reg->cal_mes_cur_high_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_HIGH_ADD1\t\t = %d\n", p_reg->cal_mes_cur_high_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_MUL0, 1, &p_reg->cal_mes_vlt_low_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_LOW_MUL0\t\t = %d\n", p_reg->cal_mes_vlt_low_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_ADD0, 1, &p_reg->cal_mes_vlt_low_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_LOW_ADD0\t\t = %d\n", p_reg->cal_mes_vlt_low_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_MUL1, 1, &p_reg->cal_mes_vlt_low_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_LOW_MUL1\t\t = %d\n", p_reg->cal_mes_vlt_low_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_ADD1, 1, &p_reg->cal_mes_vlt_low_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_LOW_ADD1\t\t = %d\n", p_reg->cal_mes_vlt_low_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_MUL0, 1, &p_reg->cal_mes_vlt_high_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_HIGH_MUL0\t\t = %d\n", p_reg->cal_mes_vlt_high_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_ADD0, 1, &p_reg->cal_mes_vlt_high_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_HIGH_ADD0\t\t = %d\n", p_reg->cal_mes_vlt_high_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_MUL1, 1, &p_reg->cal_mes_vlt_high_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_HIGH_MUL1\t\t = %d\n", p_reg->cal_mes_vlt_high_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_ADD1, 1, &p_reg->cal_mes_vlt_high_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_HIGH_ADD1\t\t = %d\n", p_reg->cal_mes_vlt_high_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_SET_CNV_POT_MUL, 1, &p_reg->cal_set_cnv_pot_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_SET_CNV_POT_MUL\t\t = %d\n", p_reg->cal_set_cnv_pot_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_SET_CNV_POT_ADD, 1, &p_reg->cal_set_cnv_pot_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_SET_CNV_POT_ADD\t\t = %d\n", p_reg->cal_set_cnv_pot_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_CNV_MUL, 1, &p_reg->cal_mes_tmp_cnv_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_CNV_MUL\t\t = %d\n", p_reg->cal_mes_tmp_cnv_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_CNV_ADD, 1, &p_reg->cal_mes_tmp_cnv_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_CNV_ADD\t\t = %d\n", p_reg->cal_mes_tmp_cnv_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_DRV_MUL, 1, &p_reg->cal_mes_tmp_drv_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_DRV_MUL\t\t = %d\n", p_reg->cal_mes_tmp_drv_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_DRV_ADD, 1, &p_reg->cal_mes_tmp_drv_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_DRV_ADD\t\t = %d\n", p_reg->cal_mes_tmp_drv_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_MUL0, 1, &p_reg->cal_mes_tmp_rem_mul0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_REM0_MUL\t\t = %d\n", p_reg->cal_mes_tmp_rem_mul0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_ADD0, 1, &p_reg->cal_mes_tmp_rem_add0, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_REM0_ADD\t\t = %d\n", p_reg->cal_mes_tmp_rem_add0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_MUL1, 1, &p_reg->cal_mes_tmp_rem_mul1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_REM1_MUL\t\t = %d\n", p_reg->cal_mes_tmp_rem_mul1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_ADD1, 1, &p_reg->cal_mes_tmp_rem_add1, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_TMP_REM1_ADD\t\t = %d\n", p_reg->cal_mes_tmp_rem_add1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_SUP_MUL, 1, &p_reg->cal_mes_vlt_sup_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_SUP_MUL\t\t = %d\n", p_reg->cal_mes_vlt_sup_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_SUP_ADD, 1, &p_reg->cal_mes_vlt_sup_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_SUP_ADD\t\t = %d\n", p_reg->cal_mes_vlt_sup_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_CNV_MUL, 1, &p_reg->cal_mes_vlt_cnv_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_CNV_MUL\t\t = %d\n", p_reg->cal_mes_vlt_cnv_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_CNV_ADD, 1, &p_reg->cal_mes_vlt_cnv_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_CNV_ADD\t\t = %d\n", p_reg->cal_mes_vlt_cnv_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_DRV_MUL, 1, &p_reg->cal_mes_vlt_drv_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_DRV_MUL\t\t = %d\n", p_reg->cal_mes_vlt_drv_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_DRV_ADD, 1, &p_reg->cal_mes_vlt_drv_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_VLT_DRV_ADD\t\t = %d\n", p_reg->cal_mes_vlt_drv_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_SUP_MUL, 1, &p_reg->cal_mes_cur_sup_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_SUP_MUL\t\t = %d\n", p_reg->cal_mes_cur_sup_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_SUP_ADD, 1, &p_reg->cal_mes_cur_sup_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_SUP_ADD\t\t = %d\n", p_reg->cal_mes_cur_sup_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_CNV_MUL, 1, &p_reg->cal_mes_cur_cnv_mul, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_CNV_MUL\t\t = %d\n", p_reg->cal_mes_cur_cnv_mul);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_CNV_ADD, 1, &p_reg->cal_mes_cur_cnv_add, 1)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_MES_CUR_CNV_ADD\t\t = %d\n", p_reg->cal_mes_cur_cnv_add);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_UNLOCK_CODE0, 1, &p_reg->cal_unlock_code0, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_UNLOCK_CODE0\t\t = 0x%04X\n", p_reg->cal_unlock_code0);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_CAL_UNLOCK_CODE1, 1, &p_reg->cal_unlock_code1, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("CAL_UNLOCK_CODE1\t\t = 0x%04X\n", p_reg->cal_unlock_code1);

	if (read_all_regs && (err = modbus_reg_rd_group(p_modbus, LTDVE2CH_ADR_BOARD_COMMAND, 1, &p_reg->board_command, 0)) != 0) {
		printf("error in modbus_reg_rd_group() call\n");
		goto exit;
	}
	if (read_all_regs && print) printf("BOARD_COMMAND\t\t\t = %d\n", p_reg->board_command);
	
exit:
	return err;
}

/* write all the general purpose registers */
int ltdve2ch_gen_reg_wr(modbus_t *p_modbus, ltdve2ch_reg_wr_t *p_reg, int print, int write_cal_regs)
{
	int err = 0;

	if (p_modbus == NULL) { err = 1; printf("bad parameter p_modbus\n"); goto exit; }
	if (p_reg == NULL) { err = 1; printf("bad parameter p_reg\n"); goto exit; }

	p_reg->osc_period = FORCE_RANGE(p_reg->osc_period, 0, 65535);
	p_reg->filter_sel0 = FORCE_RANGE(p_reg->filter_sel0, 0, 65535);
	p_reg->filter_sel1 = FORCE_RANGE(p_reg->filter_sel1, 0, 65535);
	p_reg->input_sel0 = FORCE_RANGE(p_reg->input_sel0, 0, 65535);
	p_reg->input_sel1 = FORCE_RANGE(p_reg->input_sel1, 0, 65535);
	p_reg->input_sel2 = FORCE_RANGE(p_reg->input_sel2, 0, 65535);
	p_reg->input_sel3 = FORCE_RANGE(p_reg->input_sel3, 0, 65535);
	p_reg->gen_dly_base0 = FORCE_RANGE(p_reg->gen_dly_base0, 0, 65535);
	p_reg->gen_dly_cnt0 = FORCE_RANGE(p_reg->gen_dly_cnt0, 0, 65535);
	p_reg->gen_wdt_base0 = FORCE_RANGE(p_reg->gen_wdt_base0, 0, 65535);
	p_reg->gen_wdt_cnt0 = FORCE_RANGE(p_reg->gen_wdt_cnt0, 0, 65535);
	p_reg->gen_dly_base1 = FORCE_RANGE(p_reg->gen_dly_base1, 0, 65535);
	p_reg->gen_dly_cnt1 = FORCE_RANGE(p_reg->gen_dly_cnt1, 0, 65535);
	p_reg->gen_wdt_base1 = FORCE_RANGE(p_reg->gen_wdt_base1, 0, 65535);
	p_reg->gen_wdt_cnt1 = FORCE_RANGE(p_reg->gen_wdt_cnt1, 0, 65535);
	p_reg->gen_dly_base2 = FORCE_RANGE(p_reg->gen_dly_base2, 0, 65535);
	p_reg->gen_dly_cnt2 = FORCE_RANGE(p_reg->gen_dly_cnt2, 0, 65535);
	p_reg->gen_wdt_base2 = FORCE_RANGE(p_reg->gen_wdt_base2, 0, 65535);
	p_reg->gen_wdt_cnt2 = FORCE_RANGE(p_reg->gen_wdt_cnt2, 0, 65535);
	p_reg->gen_dly_base3 = FORCE_RANGE(p_reg->gen_dly_base3, 0, 65535);
	p_reg->gen_dly_cnt3 = FORCE_RANGE(p_reg->gen_dly_cnt3, 0, 65535);
	p_reg->gen_wdt_base3 = FORCE_RANGE(p_reg->gen_wdt_base3, 0, 65535);
	p_reg->gen_wdt_cnt3 = FORCE_RANGE(p_reg->gen_wdt_cnt3, 0, 65535);
	p_reg->output_sel_hi0 = FORCE_RANGE(p_reg->output_sel_hi0, 0, 65535);
	p_reg->output_sel_lo0 = FORCE_RANGE(p_reg->output_sel_lo0, 0, 65535);
	p_reg->output_sel_hi1 = FORCE_RANGE(p_reg->output_sel_hi1, 0, 65535);
	p_reg->output_sel_lo1 = FORCE_RANGE(p_reg->output_sel_lo1, 0, 65535);
	p_reg->output_sel_hi8 = FORCE_RANGE(p_reg->output_sel_hi8, 0, 65535);
	p_reg->output_sel_lo8 = FORCE_RANGE(p_reg->output_sel_lo8, 0, 65535);
	p_reg->output_sel_hi9 = FORCE_RANGE(p_reg->output_sel_hi9, 0, 65535);
	p_reg->output_sel_lo9 = FORCE_RANGE(p_reg->output_sel_lo9, 0, 65535);
	p_reg->prt_cnt_on0 = FORCE_RANGE(p_reg->prt_cnt_on0, 0, 65535);
	p_reg->prt_ena_on0 = FORCE_RANGE(p_reg->prt_ena_on0, 0, 65535);
	p_reg->prt_cnt_off0 = FORCE_RANGE(p_reg->prt_cnt_off0, 0, 65535);
	p_reg->prt_ena_off0 = FORCE_RANGE(p_reg->prt_ena_off0, 0, 65535);
	p_reg->prt_cnt_on1 = FORCE_RANGE(p_reg->prt_cnt_on1, 0, 65535);
	p_reg->prt_ena_on1 = FORCE_RANGE(p_reg->prt_ena_on1, 0, 65535);
	p_reg->prt_cnt_off1 = FORCE_RANGE(p_reg->prt_cnt_off1, 0, 65535);
	p_reg->prt_ena_off1 = FORCE_RANGE(p_reg->prt_ena_off1, 0, 65535);
	p_reg->cur_range0 = FORCE_RANGE(p_reg->cur_range0, 0, 65535);
	p_reg->cur_value0 = FORCE_RANGE(p_reg->cur_value0, 0, 65535);
	p_reg->cur_range1 = FORCE_RANGE(p_reg->cur_range1, 0, 65535);
	p_reg->cur_value1 = FORCE_RANGE(p_reg->cur_value1, 0, 65535);
	p_reg->rs485_modbus_addr = FORCE_RANGE(p_reg->rs485_modbus_addr, 1, 247);
	p_reg->rs485_line_speed = FORCE_RANGE(p_reg->rs485_line_speed, 0, 65535);
	p_reg->rs485_line_parity = FORCE_RANGE(p_reg->rs485_line_parity, 0, 65535);
	p_reg->eth_mac_addr0 = FORCE_RANGE(p_reg->eth_mac_addr0, 0, 65535);
	p_reg->eth_mac_addr1 = FORCE_RANGE(p_reg->eth_mac_addr1, 0, 65535);
	p_reg->eth_mac_addr2 = FORCE_RANGE(p_reg->eth_mac_addr2, 0, 65535);
	p_reg->eth_hostname0 = FORCE_RANGE(p_reg->eth_hostname0, 0, 65535);
	p_reg->eth_hostname1 = FORCE_RANGE(p_reg->eth_hostname1, 0, 65535);
	p_reg->eth_hostname2 = FORCE_RANGE(p_reg->eth_hostname2, 0, 65535);
	p_reg->eth_hostname3 = FORCE_RANGE(p_reg->eth_hostname3, 0, 65535);
	p_reg->eth_hostname4 = FORCE_RANGE(p_reg->eth_hostname4, 0, 65535);
	p_reg->eth_hostname5 = FORCE_RANGE(p_reg->eth_hostname5, 0, 65535);
	p_reg->eth_hostname6 = FORCE_RANGE(p_reg->eth_hostname6, 0, 65535);
	p_reg->eth_hostname7 = FORCE_RANGE(p_reg->eth_hostname7, 0, 65535);
	p_reg->eth_dhcp_enable = FORCE_RANGE(p_reg->eth_dhcp_enable, 0, 65535);
	p_reg->eth_ip_addr_hi = FORCE_RANGE(p_reg->eth_ip_addr_hi, 0, 65535);
	p_reg->eth_ip_addr_lo = FORCE_RANGE(p_reg->eth_ip_addr_lo, 0, 65535);
	p_reg->eth_subnet_mask_hi = FORCE_RANGE(p_reg->eth_subnet_mask_hi, 0, 65535);
	p_reg->eth_subnet_mask_lo = FORCE_RANGE(p_reg->eth_subnet_mask_lo, 0, 65535);
	p_reg->eth_def_gateway_hi = FORCE_RANGE(p_reg->eth_def_gateway_hi, 0, 65535);
	p_reg->eth_def_gateway_lo = FORCE_RANGE(p_reg->eth_def_gateway_lo, 0, 65535);
	p_reg->eth_pri_dns_hi = FORCE_RANGE(p_reg->eth_pri_dns_hi, 0, 65535);
	p_reg->eth_pri_dns_lo = FORCE_RANGE(p_reg->eth_pri_dns_lo, 0, 65535);
	p_reg->eth_sec_dns_hi = FORCE_RANGE(p_reg->eth_sec_dns_hi, 0, 65535);
	p_reg->eth_sec_dns_lo = FORCE_RANGE(p_reg->eth_sec_dns_lo, 0, 65535);
	p_reg->eth_modbus_addr = FORCE_RANGE(p_reg->eth_modbus_addr, 1, 247);
	p_reg->eth_modbus_tcp_port = FORCE_RANGE(p_reg->eth_modbus_tcp_port, 1, 65535);
	p_reg->eth_modbus_udp_port = FORCE_RANGE(p_reg->eth_modbus_udp_port, 1, 65535);
	p_reg->web_password0 = FORCE_RANGE(p_reg->web_password0, 0, 65535);
	p_reg->web_password1 = FORCE_RANGE(p_reg->web_password1, 0, 65535);
	p_reg->web_password2 = FORCE_RANGE(p_reg->web_password2, 0, 65535);
	p_reg->web_password3 = FORCE_RANGE(p_reg->web_password3, 0, 65535);
	p_reg->gen_hld_base0 = FORCE_RANGE(p_reg->gen_hld_base0, 0, 65535);
	p_reg->gen_hld_cnt0 = FORCE_RANGE(p_reg->gen_hld_cnt0, 0, 65535);
	p_reg->gen_hld_base1 = FORCE_RANGE(p_reg->gen_hld_base1, 0, 65535);
	p_reg->gen_hld_cnt1 = FORCE_RANGE(p_reg->gen_hld_cnt1, 0, 65535);
	p_reg->gen_hld_base2 = FORCE_RANGE(p_reg->gen_hld_base2, 0, 65535);
	p_reg->gen_hld_cnt2 = FORCE_RANGE(p_reg->gen_hld_cnt2, 0, 65535);
	p_reg->gen_hld_base3 = FORCE_RANGE(p_reg->gen_hld_base3, 0, 65535);
	p_reg->gen_hld_cnt3 = FORCE_RANGE(p_reg->gen_hld_cnt3, 0, 65535);
	p_reg->gen_edge_sel0 = FORCE_RANGE(p_reg->gen_edge_sel0, 0, 65535);
	p_reg->gen_edge_sel1 = FORCE_RANGE(p_reg->gen_edge_sel1, 0, 65535);
	p_reg->gen_edge_sel2 = FORCE_RANGE(p_reg->gen_edge_sel2, 0, 65535);
	p_reg->gen_edge_sel3 = FORCE_RANGE(p_reg->gen_edge_sel3, 0, 65535);
	p_reg->cur_red_delay0 = FORCE_RANGE(p_reg->cur_red_delay0, 0, 65535);
	p_reg->cur_red_value0 = FORCE_RANGE(p_reg->cur_red_value0, 0, 65535);
	p_reg->cur_red_delay1 = FORCE_RANGE(p_reg->cur_red_delay1, 0, 65535);
	p_reg->cur_red_value1 = FORCE_RANGE(p_reg->cur_red_value1, 0, 65535);
	p_reg->software_trigger = FORCE_RANGE(p_reg->software_trigger, 0, 65535);
	p_reg->converter_mode = FORCE_RANGE(p_reg->converter_mode, 0, 65535);
	p_reg->converter_maximum_voltage = FORCE_RANGE(p_reg->converter_maximum_voltage, 0, 65535);
	p_reg->driver_mode = FORCE_RANGE(p_reg->driver_mode, 0, 65535);
	p_reg->inp_pol_sel0 = FORCE_RANGE(p_reg->inp_pol_sel0, 0, 1);
	p_reg->inp_pol_sel1 = FORCE_RANGE(p_reg->inp_pol_sel1, 0, 1);
	p_reg->cal_gen_cur_low_mul0 = FORCE_RANGE(p_reg->cal_gen_cur_low_mul0, -32768, 32767);
	p_reg->cal_gen_cur_low_add0 = FORCE_RANGE(p_reg->cal_gen_cur_low_add0, -32768, 32767);
	p_reg->cal_gen_cur_low_mul1 = FORCE_RANGE(p_reg->cal_gen_cur_low_mul1, -32768, 32767);
	p_reg->cal_gen_cur_low_add1 = FORCE_RANGE(p_reg->cal_gen_cur_low_add1, -32768, 32767);
	p_reg->cal_gen_cur_high_mul0 = FORCE_RANGE(p_reg->cal_gen_cur_high_mul0, -32768, 32767);
	p_reg->cal_gen_cur_high_add0 = FORCE_RANGE(p_reg->cal_gen_cur_high_add0, -32768, 32767);
	p_reg->cal_gen_cur_high_mul1 = FORCE_RANGE(p_reg->cal_gen_cur_high_mul1, -32768, 32767);
	p_reg->cal_gen_cur_high_add1 = FORCE_RANGE(p_reg->cal_gen_cur_high_add1, -32768, 32767);
	p_reg->cal_mes_cur_low_mul0 = FORCE_RANGE(p_reg->cal_mes_cur_low_mul0, -32768, 32767);
	p_reg->cal_mes_cur_low_add0 = FORCE_RANGE(p_reg->cal_mes_cur_low_add0, -32768, 32767);
	p_reg->cal_mes_cur_low_mul1 = FORCE_RANGE(p_reg->cal_mes_cur_low_mul1, -32768, 32767);
	p_reg->cal_mes_cur_low_add1 = FORCE_RANGE(p_reg->cal_mes_cur_low_add1, -32768, 32767);
	p_reg->cal_mes_cur_high_mul0 = FORCE_RANGE(p_reg->cal_mes_cur_high_mul0, -32768, 32767);
	p_reg->cal_mes_cur_high_add0 = FORCE_RANGE(p_reg->cal_mes_cur_high_add0, -32768, 32767);
	p_reg->cal_mes_cur_high_mul1 = FORCE_RANGE(p_reg->cal_mes_cur_high_mul1, -32768, 32767);
	p_reg->cal_mes_cur_high_add1 = FORCE_RANGE(p_reg->cal_mes_cur_high_add1, -32768, 32767);
	p_reg->cal_mes_vlt_low_mul0 = FORCE_RANGE(p_reg->cal_mes_vlt_low_mul0, -32768, 32767);
	p_reg->cal_mes_vlt_low_add0 = FORCE_RANGE(p_reg->cal_mes_vlt_low_add0, -32768, 32767);
	p_reg->cal_mes_vlt_low_mul1 = FORCE_RANGE(p_reg->cal_mes_vlt_low_mul1, -32768, 32767);
	p_reg->cal_mes_vlt_low_add1 = FORCE_RANGE(p_reg->cal_mes_vlt_low_add1, -32768, 32767);
	p_reg->cal_mes_vlt_high_mul0 = FORCE_RANGE(p_reg->cal_mes_vlt_high_mul0, -32768, 32767);
	p_reg->cal_mes_vlt_high_add0 = FORCE_RANGE(p_reg->cal_mes_vlt_high_add0, -32768, 32767);
	p_reg->cal_mes_vlt_high_mul1 = FORCE_RANGE(p_reg->cal_mes_vlt_high_mul1, -32768, 32767);
	p_reg->cal_mes_vlt_high_add1 = FORCE_RANGE(p_reg->cal_mes_vlt_high_add1, -32768, 32767);
	p_reg->cal_set_cnv_pot_mul = FORCE_RANGE(p_reg->cal_set_cnv_pot_mul, -32768, 32767);
	p_reg->cal_set_cnv_pot_add = FORCE_RANGE(p_reg->cal_set_cnv_pot_add, -32768, 32767);
	p_reg->cal_mes_tmp_cnv_mul = FORCE_RANGE(p_reg->cal_mes_tmp_cnv_mul, -32768, 32767);
	p_reg->cal_mes_tmp_cnv_add = FORCE_RANGE(p_reg->cal_mes_tmp_cnv_add, -32768, 32767);
	p_reg->cal_mes_tmp_drv_mul = FORCE_RANGE(p_reg->cal_mes_tmp_drv_mul, -32768, 32767);
	p_reg->cal_mes_tmp_drv_add = FORCE_RANGE(p_reg->cal_mes_tmp_drv_add, -32768, 32767);
	p_reg->cal_mes_tmp_rem_mul0 = FORCE_RANGE(p_reg->cal_mes_tmp_rem_mul0, -32768, 32767);
	p_reg->cal_mes_tmp_rem_add0 = FORCE_RANGE(p_reg->cal_mes_tmp_rem_add0, -32768, 32767);
	p_reg->cal_mes_tmp_rem_mul1 = FORCE_RANGE(p_reg->cal_mes_tmp_rem_mul1, -32768, 32767);
	p_reg->cal_mes_tmp_rem_add1 = FORCE_RANGE(p_reg->cal_mes_tmp_rem_add1, -32768, 32767);
	p_reg->cal_mes_vlt_sup_mul = FORCE_RANGE(p_reg->cal_mes_vlt_sup_mul, -32768, 32767);
	p_reg->cal_mes_vlt_sup_add = FORCE_RANGE(p_reg->cal_mes_vlt_sup_add, -32768, 32767);
	p_reg->cal_mes_vlt_cnv_mul = FORCE_RANGE(p_reg->cal_mes_vlt_cnv_mul, -32768, 32767);
	p_reg->cal_mes_vlt_cnv_add = FORCE_RANGE(p_reg->cal_mes_vlt_cnv_add, -32768, 32767);
	p_reg->cal_mes_vlt_drv_mul = FORCE_RANGE(p_reg->cal_mes_vlt_drv_mul, -32768, 32767);
	p_reg->cal_mes_vlt_drv_add = FORCE_RANGE(p_reg->cal_mes_vlt_drv_add, -32768, 32767);
	p_reg->cal_mes_cur_sup_mul = FORCE_RANGE(p_reg->cal_mes_cur_sup_mul, -32768, 32767);
	p_reg->cal_mes_cur_sup_add = FORCE_RANGE(p_reg->cal_mes_cur_sup_add, -32768, 32767);
	p_reg->cal_mes_cur_cnv_mul = FORCE_RANGE(p_reg->cal_mes_cur_cnv_mul, -32768, 32767);
	p_reg->cal_mes_cur_cnv_add = FORCE_RANGE(p_reg->cal_mes_cur_cnv_add, -32768, 32767);
	p_reg->cal_unlock_code0 = FORCE_RANGE(p_reg->cal_unlock_code0, 0, 65535);
	p_reg->cal_unlock_code1 = FORCE_RANGE(p_reg->cal_unlock_code1, 0, 65535);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OSC_PERIOD, 1, &p_reg->osc_period)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OSC_PERIOD\t\t\t = %d\n", p_reg->osc_period);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_FILTER_SEL0, 1, &p_reg->filter_sel0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("FILTER_SEL0\t\t\t = %d\n", p_reg->filter_sel0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_FILTER_SEL1, 1, &p_reg->filter_sel1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("FILTER_SEL1\t\t\t = %d\n", p_reg->filter_sel1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL0, 1, &p_reg->input_sel0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("INPUT_SEL0\t\t\t = 0x%04X\n", p_reg->input_sel0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL1, 1, &p_reg->input_sel1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("INPUT_SEL1\t\t\t = 0x%04X\n", p_reg->input_sel1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL2, 1, &p_reg->input_sel2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("INPUT_SEL2\t\t\t = 0x%04X\n", p_reg->input_sel2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_INPUT_SEL3, 1, &p_reg->input_sel3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("INPUT_SEL3\t\t\t = 0x%04X\n", p_reg->input_sel3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE0, 1, &p_reg->gen_dly_base0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_BASE0\t\t\t = %d\n", p_reg->gen_dly_base0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT0, 1, &p_reg->gen_dly_cnt0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_COUNT0\t\t = %d\n", p_reg->gen_dly_cnt0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE0, 1, &p_reg->gen_wdt_base0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_BASE0\t\t\t = %d\n", p_reg->gen_wdt_base0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT0, 1, &p_reg->gen_wdt_cnt0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_COUNT0\t\t = %d\n", p_reg->gen_wdt_cnt0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE1, 1, &p_reg->gen_dly_base1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_BASE1\t\t\t = %d\n", p_reg->gen_dly_base1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT1, 1, &p_reg->gen_dly_cnt1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_COUNT1\t\t = %d\n", p_reg->gen_dly_cnt1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE1, 1, &p_reg->gen_wdt_base1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_BASE1\t\t\t = %d\n", p_reg->gen_wdt_base1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT1, 1, &p_reg->gen_wdt_cnt1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_COUNT1\t\t = %d\n", p_reg->gen_wdt_cnt1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE2, 1, &p_reg->gen_dly_base2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_BASE2\t\t\t = %d\n", p_reg->gen_dly_base2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT2, 1, &p_reg->gen_dly_cnt2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_COUNT2\t\t = %d\n", p_reg->gen_dly_cnt2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE2, 1, &p_reg->gen_wdt_base2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_BASE2\t\t\t = %d\n", p_reg->gen_wdt_base2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT2, 1, &p_reg->gen_wdt_cnt2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_COUNT2\t\t = %d\n", p_reg->gen_wdt_cnt2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_BASE3, 1, &p_reg->gen_dly_base3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_BASE3\t\t\t = %d\n", p_reg->gen_dly_base3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_DLY_CNT3, 1, &p_reg->gen_dly_cnt3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_DELAY_COUNT3\t\t = %d\n", p_reg->gen_dly_cnt3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_BASE3, 1, &p_reg->gen_wdt_base3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_BASE3\t\t\t = %d\n", p_reg->gen_wdt_base3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_WDT_CNT3, 1, &p_reg->gen_wdt_cnt3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_WIDTH_COUNT3\t\t = %d\n", p_reg->gen_wdt_cnt3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI0, 1, &p_reg->output_sel_hi0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_HI0\t\t\t = 0x%04X\n", p_reg->output_sel_hi0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO0, 1, &p_reg->output_sel_lo0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_LO0\t\t\t = 0x%04X\n", p_reg->output_sel_lo0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI1, 1, &p_reg->output_sel_hi1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_HI1\t\t\t = 0x%04X\n", p_reg->output_sel_hi1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO1, 1, &p_reg->output_sel_lo1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_LO1\t\t\t = 0x%04X\n", p_reg->output_sel_lo1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI8, 1, &p_reg->output_sel_hi8)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_HI8\t\t\t = 0x%04X\n", p_reg->output_sel_hi8);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO8, 1, &p_reg->output_sel_lo8)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_LO8\t\t\t = 0x%04X\n", p_reg->output_sel_lo8);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_HI9, 1, &p_reg->output_sel_hi9)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_HI9\t\t\t = 0x%04X\n", p_reg->output_sel_hi9);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_OUTPUT_SEL_LO9, 1, &p_reg->output_sel_lo9)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("OUTPUT_SEL_LO9\t\t\t = 0x%04X\n", p_reg->output_sel_lo9);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_ON0, 1, &p_reg->prt_cnt_on0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_CNT_ON0\t\t\t = %d\n", p_reg->prt_cnt_on0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_ON0, 1, &p_reg->prt_ena_on0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_ENA_ON0\t\t\t = %d\n", p_reg->prt_ena_on0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_OFF0, 1, &p_reg->prt_cnt_off0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_CNT_OFF0\t\t\t = %d\n", p_reg->prt_cnt_off0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_OFF0, 1, &p_reg->prt_ena_off0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_ENA_OFF0\t\t\t = %d\n", p_reg->prt_ena_off0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_ON1, 1, &p_reg->prt_cnt_on1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_CNT_ON1\t\t\t = %d\n", p_reg->prt_cnt_on1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_ON1, 1, &p_reg->prt_ena_on1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_ENA_ON1\t\t\t = %d\n", p_reg->prt_ena_on1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_CNT_OFF1, 1, &p_reg->prt_cnt_off1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_CNT_OFF1\t\t\t = %d\n", p_reg->prt_cnt_off1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_PRT_ENA_OFF1, 1, &p_reg->prt_ena_off1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("PRT_ENA_OFF1\t\t\t = %d\n", p_reg->prt_ena_off1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_RANGE0, 1, &p_reg->cur_range0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_RANGE0\t\t\t = %d\n", p_reg->cur_range0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_VALUE0, 1, &p_reg->cur_value0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_VALUE0\t\t\t = %d => %.3f A\n", p_reg->cur_value0, p_reg->cur_value0 / 1000.0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_RANGE1, 1, &p_reg->cur_range1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_RANGE1\t\t\t = %d\n", p_reg->cur_range1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_VALUE1, 1, &p_reg->cur_value1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_VALUE1\t\t\t = %d => %.3f A\n", p_reg->cur_value1, p_reg->cur_value1 / 1000.0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_RS485_MODBUS_ADDR, 1, &p_reg->rs485_modbus_addr)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("RS485_MODBUS_ADDR\t\t = %d\n", p_reg->rs485_modbus_addr);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_RS485_LINE_SPEED, 1, &p_reg->rs485_line_speed)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("RS485_LINE_SPEED\t\t = %d\n", p_reg->rs485_line_speed);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_RS485_LINE_PARITY, 1, &p_reg->rs485_line_parity)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("RS485_LINE_PARITY\t\t = %d\n", p_reg->rs485_line_parity);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_MAC_ADDR0, 1, &p_reg->eth_mac_addr0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("ETH_MAC_ADDR0\t\t\t = 0x%04X\n", p_reg->eth_mac_addr0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_MAC_ADDR1, 1, &p_reg->eth_mac_addr1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("ETH_MAC_ADDR1\t\t\t = 0x%04X\n", p_reg->eth_mac_addr1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_MAC_ADDR2, 1, &p_reg->eth_mac_addr2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("ETH_MAC_ADDR2\t\t\t = 0x%04X\n", p_reg->eth_mac_addr2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME0, 1, &p_reg->eth_hostname0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME0\t\t\t = 0x%04X\n", p_reg->eth_hostname0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME1, 1, &p_reg->eth_hostname1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME1\t\t\t = 0x%04X\n", p_reg->eth_hostname1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME2, 1, &p_reg->eth_hostname2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME2\t\t\t = 0x%04X\n", p_reg->eth_hostname2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME3, 1, &p_reg->eth_hostname3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME3\t\t\t = 0x%04X\n", p_reg->eth_hostname3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME4, 1, &p_reg->eth_hostname4)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME4\t\t\t = 0x%04X\n", p_reg->eth_hostname4);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME5, 1, &p_reg->eth_hostname5)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME5\t\t\t = 0x%04X\n", p_reg->eth_hostname5);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME6, 1, &p_reg->eth_hostname6)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME6\t\t\t = 0x%04X\n", p_reg->eth_hostname6);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_HOSTNAME7, 1, &p_reg->eth_hostname7)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_HOSTNAME7\t\t\t = 0x%04X\n", p_reg->eth_hostname7);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_DHCP_ENABLE, 1, &p_reg->eth_dhcp_enable)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_DHCP_ENABLE\t\t\t = %d\n", p_reg->eth_dhcp_enable);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_IP_ADDR_HI, 1, &p_reg->eth_ip_addr_hi)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_IP_ADDR_HI\t\t\t = 0x%04X\n", p_reg->eth_ip_addr_hi);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_IP_ADDR_LO, 1, &p_reg->eth_ip_addr_lo)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_IP_ADDR_LO\t\t\t = 0x%04X\n", p_reg->eth_ip_addr_lo);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_SUBNET_MASK_HI, 1, &p_reg->eth_subnet_mask_hi)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_SUBNET_MASK_HI\t\t = 0x%04X\n", p_reg->eth_subnet_mask_hi);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_SUBNET_MASK_LO, 1, &p_reg->eth_subnet_mask_lo)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_SUBNET_MASK_LO\t\t = 0x%04X\n", p_reg->eth_subnet_mask_lo);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_DEF_GATEWAY_HI, 1, &p_reg->eth_def_gateway_hi)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_DEF_GATEWAY_HI\t\t = 0x%04X\n", p_reg->eth_def_gateway_hi);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_DEF_GATEWAY_LO, 1, &p_reg->eth_def_gateway_lo)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_DEF_GATEWAY_LO\t\t = 0x%04X\n", p_reg->eth_def_gateway_lo);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_PRI_DNS_HI, 1, &p_reg->eth_pri_dns_hi)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_PRI_DNS_HI\t\t\t = 0x%04X\n", p_reg->eth_pri_dns_hi);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_PRI_DNS_LO, 1, &p_reg->eth_pri_dns_lo)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_PRI_DNS_LO\t\t\t = 0x%04X\n", p_reg->eth_pri_dns_lo);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_SEC_DNS_HI, 1, &p_reg->eth_sec_dns_hi)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_SEC_DNS_HI\t\t\t = 0x%04X\n", p_reg->eth_sec_dns_hi);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_SEC_DNS_LO, 1, &p_reg->eth_sec_dns_lo)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_SEC_DNS_LO\t\t\t = 0x%04X\n", p_reg->eth_sec_dns_lo);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_MODBUS_ADDR, 1, &p_reg->eth_modbus_addr)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_MODBUS_ADDR\t\t\t = %d\n", p_reg->eth_modbus_addr);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_MODBUS_TCP_PORT, 1, &p_reg->eth_modbus_tcp_port)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_MODBUS_TCP_PORT\t\t = %d\n", p_reg->eth_modbus_tcp_port);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_ETH_MODBUS_UDP_PORT, 1, &p_reg->eth_modbus_udp_port)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("ETH_MODBUS_UDP_PORT\t\t = %d\n", p_reg->eth_modbus_udp_port);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD0, 1, &p_reg->web_password0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("WEB_PASSWORD0\t\t\t = 0x%04X\n", p_reg->web_password0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD1, 1, &p_reg->web_password1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("WEB_PASSWORD1\t\t\t = 0x%04X\n", p_reg->web_password1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD2, 1, &p_reg->web_password2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("WEB_PASSWORD2\t\t\t = 0x%04X\n", p_reg->web_password2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_WEB_PASSWORD3, 1, &p_reg->web_password3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("WEB_PASSWORD3\t\t\t = 0x%04X\n", p_reg->web_password3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE0, 1, &p_reg->gen_hld_base0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_BASE0\t\t\t = %d\n", p_reg->gen_hld_base0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT0, 1, &p_reg->gen_hld_cnt0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_COUNT0\t\t\t = %d\n", p_reg->gen_hld_cnt0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE1, 1, &p_reg->gen_hld_base1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_BASE1\t\t\t = %d\n", p_reg->gen_hld_base1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT1, 1, &p_reg->gen_hld_cnt1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_COUNT1\t\t\t = %d\n", p_reg->gen_hld_cnt1);
    
	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE2, 1, &p_reg->gen_hld_base2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_BASE2\t\t\t = %d\n", p_reg->gen_hld_base2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT2, 1, &p_reg->gen_hld_cnt2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_COUNT2\t\t\t = %d\n", p_reg->gen_hld_cnt2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_BASE3, 1, &p_reg->gen_hld_base3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_BASE3\t\t\t = %d\n", p_reg->gen_hld_base3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_HLD_CNT3, 1, &p_reg->gen_hld_cnt3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_HOLD_COUNT3\t\t\t = %d\n", p_reg->gen_hld_cnt3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL0, 1, &p_reg->gen_edge_sel0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_EDGE_SEL0\t\t\t = %d\n", p_reg->gen_edge_sel0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL1, 1, &p_reg->gen_edge_sel1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_EDGE_SEL1\t\t\t = %d\n", p_reg->gen_edge_sel1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL2, 1, &p_reg->gen_edge_sel2)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_EDGE_SEL2\t\t\t = %d\n", p_reg->gen_edge_sel2);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_GEN_EDGE_SEL3, 1, &p_reg->gen_edge_sel3)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("GEN_EDGE_SEL3\t\t\t = %d\n", p_reg->gen_edge_sel3);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_INP_POL_SEL0, 1, &p_reg->inp_pol_sel0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("POLARITY_SEL0\t\t\t = %d\n", p_reg->inp_pol_sel0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_INP_POL_SEL1, 1, &p_reg->inp_pol_sel1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("POLARITY_SEL1\t\t\t = %d\n", p_reg->inp_pol_sel1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_RED_DLY0, 1, &p_reg->cur_red_delay0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_RED_DELAY0\t\t\t = %d ms\n", p_reg->cur_red_delay0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_RED_VAL0, 1, &p_reg->cur_red_value0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_RED_VALUE0\t\t\t = %d %%\n", p_reg->cur_red_value0);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_RED_DLY1, 1, &p_reg->cur_red_delay1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_RED_DELAY1\t\t\t = %d ms\n", p_reg->cur_red_delay1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CUR_RED_VAL1, 1, &p_reg->cur_red_value1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CUR_RED_VALUE1\t\t\t = %d %%\n", p_reg->cur_red_value1);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_SOFTWARE_TRIGGER, 1, &p_reg->software_trigger)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("SOFTWARE_TRIGGER\t\t = %d\n", p_reg->software_trigger);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CONVERTER_MODE, 1, &p_reg->converter_mode)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CONVERTER_MODE\t\t\t = %d\n", p_reg->converter_mode);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CONVERTER_MAXIMUM_VLT, 1, &p_reg->converter_maximum_voltage)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("CONVERTER_MAXIMUM_VOLTAGE\t = %d V\n", p_reg->converter_maximum_voltage);

	if ((err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_DRIVER_MODE, 1, &p_reg->driver_mode)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (print) printf("DRIVER_MODE\t\t\t = %d\n", p_reg->driver_mode);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_MUL0, 1, &p_reg->cal_gen_cur_low_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_LOW_MUL0\t\t = %d\n", p_reg->cal_gen_cur_low_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_ADD0, 1, &p_reg->cal_gen_cur_low_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_LOW_ADD0\t\t = %d\n", p_reg->cal_gen_cur_low_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_MUL1, 1, &p_reg->cal_gen_cur_low_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_LOW_MUL1\t\t = %d\n", p_reg->cal_gen_cur_low_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_LOW_ADD1, 1, &p_reg->cal_gen_cur_low_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_LOW_ADD1\t\t = %d\n", p_reg->cal_gen_cur_low_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_MUL0, 1, &p_reg->cal_gen_cur_high_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_HIGH_MUL0\t\t = %d\n", p_reg->cal_gen_cur_high_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_ADD0, 1, &p_reg->cal_gen_cur_high_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_HIGH_ADD0\t\t = %d\n", p_reg->cal_gen_cur_high_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_MUL1, 1, &p_reg->cal_gen_cur_high_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_HIGH_MUL1\t\t = %d\n", p_reg->cal_gen_cur_high_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_GEN_CUR_HIGH_ADD1, 1, &p_reg->cal_gen_cur_high_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_GEN_CUR_HIGH_ADD1\t\t = %d\n", p_reg->cal_gen_cur_high_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_MUL0, 1, &p_reg->cal_mes_cur_low_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_LOW_MUL0\t\t = %d\n", p_reg->cal_mes_cur_low_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_ADD0, 1, &p_reg->cal_mes_cur_low_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_LOW_ADD0\t\t = %d\n", p_reg->cal_mes_cur_low_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_MUL1, 1, &p_reg->cal_mes_cur_low_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_LOW_MUL1\t\t = %d\n", p_reg->cal_mes_cur_low_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_LOW_ADD1, 1, &p_reg->cal_mes_cur_low_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_LOW_ADD1\t\t = %d\n", p_reg->cal_mes_cur_low_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_MUL0, 1, &p_reg->cal_mes_cur_high_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_HIGH_MUL0\t\t = %d\n", p_reg->cal_mes_cur_high_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_ADD0, 1, &p_reg->cal_mes_cur_high_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_HIGH_ADD0\t\t = %d\n", p_reg->cal_mes_cur_high_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_MUL1, 1, &p_reg->cal_mes_cur_high_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_HIGH_MUL1\t\t = %d\n", p_reg->cal_mes_cur_high_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_HIGH_ADD1, 1, &p_reg->cal_mes_cur_high_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_HIGH_ADD1\t\t = %d\n", p_reg->cal_mes_cur_high_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_MUL0, 1, &p_reg->cal_mes_vlt_low_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_LOW_MUL0\t\t = %d\n", p_reg->cal_mes_vlt_low_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_ADD0, 1, &p_reg->cal_mes_vlt_low_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_LOW_ADD0\t\t = %d\n", p_reg->cal_mes_vlt_low_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_MUL1, 1, &p_reg->cal_mes_vlt_low_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_LOW_MUL1\t\t = %d\n", p_reg->cal_mes_vlt_low_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_LOW_ADD1, 1, &p_reg->cal_mes_vlt_low_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_LOW_ADD1\t\t = %d\n", p_reg->cal_mes_vlt_low_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_MUL0, 1, &p_reg->cal_mes_vlt_high_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_HIGH_MUL0\t\t = %d\n", p_reg->cal_mes_vlt_high_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_ADD0, 1, &p_reg->cal_mes_vlt_high_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_HIGH_ADD0\t\t = %d\n", p_reg->cal_mes_vlt_high_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_MUL1, 1, &p_reg->cal_mes_vlt_high_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_HIGH_MUL1\t\t = %d\n", p_reg->cal_mes_vlt_high_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_HIGH_ADD1, 1, &p_reg->cal_mes_vlt_high_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_HIGH_ADD1\t\t = %d\n", p_reg->cal_mes_vlt_high_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_SET_CNV_POT_MUL, 1, &p_reg->cal_set_cnv_pot_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_SET_CNV_POT_MUL\t\t = %d\n", p_reg->cal_set_cnv_pot_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_SET_CNV_POT_ADD, 1, &p_reg->cal_set_cnv_pot_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_SET_CNV_POT_ADD\t\t = %d\n", p_reg->cal_set_cnv_pot_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_CNV_MUL, 1, &p_reg->cal_mes_tmp_cnv_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_CNV_MUL\t\t = %d\n", p_reg->cal_mes_tmp_cnv_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_CNV_ADD, 1, &p_reg->cal_mes_tmp_cnv_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_CNV_ADD\t\t = %d\n", p_reg->cal_mes_tmp_cnv_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_DRV_MUL, 1, &p_reg->cal_mes_tmp_drv_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_DRV_MUL\t\t = %d\n", p_reg->cal_mes_tmp_drv_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_DRV_ADD, 1, &p_reg->cal_mes_tmp_drv_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_DRV_ADD\t\t = %d\n", p_reg->cal_mes_tmp_drv_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_MUL0, 1, &p_reg->cal_mes_tmp_rem_mul0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_REM0_MUL\t\t = %d\n", p_reg->cal_mes_tmp_rem_mul0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_ADD0, 1, &p_reg->cal_mes_tmp_rem_add0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_REM0_ADD\t\t = %d\n", p_reg->cal_mes_tmp_rem_add0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_MUL1, 1, &p_reg->cal_mes_tmp_rem_mul1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_REM1_MUL\t\t = %d\n", p_reg->cal_mes_tmp_rem_mul1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_TMP_REM_ADD1, 1, &p_reg->cal_mes_tmp_rem_add1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_TMP_REM1_ADD\t\t = %d\n", p_reg->cal_mes_tmp_rem_add1);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_SUP_MUL, 1, &p_reg->cal_mes_vlt_sup_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_SUP_MUL\t\t = %d\n", p_reg->cal_mes_vlt_sup_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_SUP_ADD, 1, &p_reg->cal_mes_vlt_sup_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_SUP_ADD\t\t = %d\n", p_reg->cal_mes_vlt_sup_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_CNV_MUL, 1, &p_reg->cal_mes_vlt_cnv_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_CNV_MUL\t\t = %d\n", p_reg->cal_mes_vlt_cnv_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_CNV_ADD, 1, &p_reg->cal_mes_vlt_cnv_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_CNV_ADD\t\t = %d\n", p_reg->cal_mes_vlt_cnv_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_DRV_MUL, 1, &p_reg->cal_mes_vlt_drv_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_DRV_MUL\t\t = %d\n", p_reg->cal_mes_vlt_drv_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_VLT_DRV_ADD, 1, &p_reg->cal_mes_vlt_drv_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_VLT_DRV_ADD\t\t = %d\n", p_reg->cal_mes_vlt_drv_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_SUP_MUL, 1, &p_reg->cal_mes_cur_sup_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_SUP_MUL\t\t = %d\n", p_reg->cal_mes_cur_sup_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_SUP_ADD, 1, &p_reg->cal_mes_cur_sup_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_SUP_ADD\t\t = %d\n", p_reg->cal_mes_cur_sup_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_CNV_MUL, 1, &p_reg->cal_mes_cur_cnv_mul)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_CNV_MUL\t\t = %d\n", p_reg->cal_mes_cur_cnv_mul);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_MES_CUR_CNV_ADD, 1, &p_reg->cal_mes_cur_cnv_add)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_MES_CUR_CNV_ADD\t\t = %d\n", p_reg->cal_mes_cur_cnv_add);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_UNLOCK_CODE0, 1, &p_reg->cal_unlock_code0)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_UNLOCK_CODE0\t\t = 0x%04X\n", p_reg->cal_unlock_code0);

	if (write_cal_regs && (err = modbus_reg_wr_group(p_modbus, LTDVE2CH_ADR_CAL_UNLOCK_CODE1, 1, &p_reg->cal_unlock_code1)) != 0) {
		printf("error in modbus_reg_wr_group() call\n");
		goto exit;
	}
	if (write_cal_regs && print) printf("CAL_UNLOCK_CODE1\t\t = 0x%04X\n", p_reg->cal_unlock_code1);

exit:
	return err;
}

/* write the board command register */
int ltdve2ch_brd_cmd_wr(modbus_t *p_modbus, int reg_val)
{
	int err = 0;

	if (p_modbus == NULL) { err = 1; printf("bad parameter p_modbus\n"); goto exit; }
	if (reg_val < 0) { err = 1; printf("bad parameter reg_val\n"); goto exit; }
	if (reg_val > 3) { err = 1; printf("bad parameter reg_val\n"); goto exit; }

	if ((err = modbus_reg_wr_single(p_modbus, LTDVE2CH_ADR_BOARD_COMMAND, reg_val)) != 0) {
		printf("error in modbus_reg_wr_single() call\n");
		goto exit;
	}

exit:
	return err;
}

/* file ends here */
